package javax.lang.model.util;

import java.util.Collections;
import java.util.EnumSet;
import java.util.LinkedHashSet;
import java.util.Iterator;
import java.util.ArrayList;
import javax.lang.model.element.PackageElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.VariableElement;
import java.util.List;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import java.util.Set;

public class ElementFilter
{
    private static final Set<ElementKind> CONSTRUCTOR_KIND;
    private static final Set<ElementKind> FIELD_KINDS;
    private static final Set<ElementKind> METHOD_KIND;
    private static final Set<ElementKind> PACKAGE_KIND;
    private static final Set<ElementKind> TYPE_KINDS;
    
    private ElementFilter() {
    }
    
    public static List<VariableElement> fieldsIn(Iterable<? extends Element> iterable) {
        return listFilter(iterable, ElementFilter.FIELD_KINDS, VariableElement.class);
    }
    
    public static Set<VariableElement> fieldsIn(Set<? extends Element> set) {
        return setFilter(set, ElementFilter.FIELD_KINDS, VariableElement.class);
    }
    
    public static List<ExecutableElement> constructorsIn(Iterable<? extends Element> iterable) {
        return listFilter(iterable, ElementFilter.CONSTRUCTOR_KIND, ExecutableElement.class);
    }
    
    public static Set<ExecutableElement> constructorsIn(Set<? extends Element> set) {
        return setFilter(set, ElementFilter.CONSTRUCTOR_KIND, ExecutableElement.class);
    }
    
    public static List<ExecutableElement> methodsIn(Iterable<? extends Element> iterable) {
        return listFilter(iterable, ElementFilter.METHOD_KIND, ExecutableElement.class);
    }
    
    public static Set<ExecutableElement> methodsIn(Set<? extends Element> set) {
        return setFilter(set, ElementFilter.METHOD_KIND, ExecutableElement.class);
    }
    
    public static List<TypeElement> typesIn(Iterable<? extends Element> iterable) {
        return listFilter(iterable, ElementFilter.TYPE_KINDS, TypeElement.class);
    }
    
    public static Set<TypeElement> typesIn(Set<? extends Element> set) {
        return setFilter(set, ElementFilter.TYPE_KINDS, TypeElement.class);
    }
    
    public static List<PackageElement> packagesIn(Iterable<? extends Element> iterable) {
        return listFilter(iterable, ElementFilter.PACKAGE_KIND, PackageElement.class);
    }
    
    public static Set<PackageElement> packagesIn(Set<? extends Element> set) {
        return setFilter(set, ElementFilter.PACKAGE_KIND, PackageElement.class);
    }
    
    private static <E extends Element> List<E> listFilter(Iterable<? extends Element> iterable, Set<ElementKind> set, Class<E> clazz) {
        ArrayList<Object> list = new ArrayList();
        for (Element obj : iterable) {
            if (set.contains(obj.getKind())) {
                list.add(clazz.cast(obj));
            }
            else {
                continue;
            }
        }
        return list;
    }
    
    private static <E extends Element> Set<E> setFilter(Set<? extends Element> set, Set<ElementKind> set2, Class<E> clazz) {
        LinkedHashSet<Object> set3 = new LinkedHashSet();
        for (Element obj : set) {
            if (set2.contains(obj.getKind())) {
                set3.add(clazz.cast(obj));
            }
            else {
                continue;
            }
        }
        return set3;
    }
    
    static {
        CONSTRUCTOR_KIND = Collections.unmodifiableSet(EnumSet.of(ElementKind.CONSTRUCTOR));
        FIELD_KINDS = Collections.unmodifiableSet(EnumSet.of(ElementKind.FIELD, ElementKind.ENUM_CONSTANT));
        METHOD_KIND = Collections.unmodifiableSet(EnumSet.of(ElementKind.METHOD));
        PACKAGE_KIND = Collections.unmodifiableSet(EnumSet.of(ElementKind.PACKAGE));
        TYPE_KINDS = Collections.unmodifiableSet(EnumSet.of(ElementKind.CLASS, ElementKind.ENUM, ElementKind.INTERFACE, ElementKind.ANNOTATION_TYPE));
    }
}
